package org.expeditee.agents;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.expeditee.agents.mail.MailSession;
import org.expeditee.gui.AttributeValuePair;
import org.expeditee.gui.Frame;
import org.expeditee.gui.FreeItems;
import org.expeditee.items.Item;
import org.expeditee.items.Text;

public class MailFrame extends DefaultAgent {

	@Override
	public boolean initialise(Frame frame, Item item) {
		return super.initialise(frame, item);
	}

	/**
	 * Email's a frame. The email addy's the mail will be sent to are taken from
	 * the frame tags of the form \@to:<AddressList>, \@cc:<AddressList> or
	 * \@bcc:<AddressList>. If such frame tags do not exist the agent looks for
	 * items on the end of the cursor. Different types of addresses can be
	 * grouped in boxes with a corner labelled, cc, bc or to. If a box does not
	 * have a label it is assumed to contain SendTo addresses.
	 */
	@Override
	protected Frame process(Frame frame) {
		String subject = frame.getTitle();
		List<Text> text = frame.getBodyTextItems(false);
		StringBuilder body = new StringBuilder();
		for (Text t : text) {
			body.append(t.getText()).append("\n\n");
		}
		String to = frame.getAnnotationValue("to");
		String cc = frame.getAnnotationValue("cc");
		String bcc = frame.getAnnotationValue("bcc");

		// Check for the address on the end of the cursor
		if (to == null) {
			StringBuffer ccString = new StringBuffer();
			StringBuffer toString = new StringBuffer();
			StringBuffer bccString = new StringBuffer();
			// Check for a group of text items attached to the cursor
			Map<String, Collection<String>> groupedText = FreeItems
					.getGroupedText();
			Collection<String> ccStrings = groupedText.get("cc");
			Collection<String> toStrings = groupedText.get("to");
			Collection<String> bccStrings = groupedText.get("bcc");

			if (cc == null && ccStrings != null) {
				for (String address : ccStrings) {
					if (address.charAt(0) == (AttributeValuePair.ANNOTATION_CHAR))
						address = address.substring(1);
					ccString.append(address);
					ccString.append(',');
				}
			}

			if (bcc == null && bccStrings != null) {
				for (String address : bccStrings) {
					if (address.charAt(0) == (AttributeValuePair.ANNOTATION_CHAR))
						address = address.substring(1);
					bccString.append(address);
					bccString.append(',');
				}
			}

			if (toStrings != null) {
				for (String address : toStrings) {
					if (address.charAt(0) == (AttributeValuePair.ANNOTATION_CHAR))
						address = address.substring(1);
					toString.append(address);
					toString.append(',');
				}
			} else {
				Collection<String> otherStrings = groupedText.get("");
				for (String address : otherStrings) {
					if (address.charAt(0) == (AttributeValuePair.ANNOTATION_CHAR))
						address = address.substring(1);
					if (address.startsWith("cc:") && address.length() > 5) {
						ccString.append(address.substring(3).trim());
						ccString.append(',');
					} else if (address.startsWith("bcc:")
							&& address.length() > 6) {
						bccString.append(address.substring(4).trim());
						bccString.append(',');
					} else {
						toString.append(address);
						toString.append(',');
					}
				}
			}
			if (toString.length() > 0) {
				toString.deleteCharAt(toString.length() - 1);
				to = toString.toString();
			}

			if (ccString.length() > 0) {
				ccString.deleteCharAt(ccString.length() - 1);
				cc = ccString.toString();
			}

			if (bccString.length() > 0) {
				bccString.deleteCharAt(ccString.length() - 1);
				bcc = bccString.toString();
			}
		}

		// Last chance for the user to stop
		if (_stop)
			return null;
		// Allow the user to do other stuff while the message gets sent
		_running = false;
		MailSession.sendTextMessage(to, cc, bcc, subject, body.toString());

		return null;
	}
}
