package org.expeditee.agents;

import java.awt.Dimension;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;

import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameGraphics;
import org.expeditee.gui.FrameIO;
import org.expeditee.gui.UserSettings;

import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfWriter;

public class PdfFrameset extends DefaultAgent {
	private long _firstFrame = 1;

	private long _maxFrame = Integer.MAX_VALUE;

	public PdfFrameset(long firstFrame, long maxFrame) {
		super();
		_firstFrame = firstFrame;
		_maxFrame = maxFrame;
	}

	public PdfFrameset() {
		super();
		_firstFrame = 1;
		_maxFrame = Integer.MAX_VALUE;
	}

	@Override
	protected Frame process(Frame frame) {
		String framesetName = frame.getFramesetName();
		String fileName = FrameIO.EXPORTS_DIR + framesetName + ".pdf";
		int count = FrameIO.getLastNumber(framesetName);

		Dimension d = FrameGraphics.getMaxSize();
		Document _pdfDocument = new Document(new Rectangle(d.width, d.height));
		
		PdfContentByte cb = null;
		try {
			PdfWriter writer = PdfWriter.getInstance(_pdfDocument,
					new FileOutputStream(fileName));
			_pdfDocument.open();
			_pdfDocument.addCreationDate();
			_pdfDocument.addAuthor(UserSettings.Username);
			_pdfDocument.addCreator("Expeditee");
			cb = writer.getDirectContent();

		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return null;
		} catch (DocumentException e) {
			e.printStackTrace();
			return null;
		}

		for (long i = _firstFrame; i <= _maxFrame && i <= count; i++) {
			if (_stop) {
				break;
			}
			String frameName = framesetName + i;
			Frame toPaint = FrameIO.LoadFrame(frameName);
			if (toPaint != null) {
				overwriteMessage("Converting " + frameName);
				java.awt.Graphics2D g2 = cb.createGraphics(FrameGraphics
						.getMaxSize().width, FrameGraphics.getMaxSize().height);
				FrameGraphics.paintFrame(toPaint, null, false, false, g2);
				g2.dispose();
				// Move to the next page?
				_pdfDocument.setPageCount(_pdfDocument.getPageNumber() + 1);
			}
		}
		try {
			_pdfDocument.close();
		} catch (Exception e) {
			overwriteMessage("Error writing PDF document");
		}
		return null;
	}
}
